"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
const events_1 = require("events");
const path_1 = __importDefault(require("path"));
const minimist_1 = __importDefault(require("minimist"));
const Command_1 = __importDefault(require("./Command"));
const utils_1 = require("./utils");
const NAME_OF_GLOBAL_COMMAND = 'this-does-not-matter';
class CAC extends events_1.EventEmitter {
    constructor() {
        super();
        this.commands = [];
        this.globalCommand = new Command_1.default(NAME_OF_GLOBAL_COMMAND, 'The global command');
        this.globalCommand.usage('<command> [options]');
    }
    usage(text) {
        this.globalCommand.usage(text);
        return this;
    }
    command(rawName, description, config) {
        const command = new Command_1.default(rawName, description, config);
        this.commands.push(command);
        return command;
    }
    option(rawName, description, config) {
        this.globalCommand.option(rawName, description, config);
        return this;
    }
    help(callback) {
        this.globalCommand.option('-h, --help', 'Display this message');
        this.globalCommand.helpCallback = callback;
        return this;
    }
    version(version, customFlags = '-v, --version') {
        this.globalCommand.version(version, customFlags);
        return this;
    }
    /**
     * Add a global example
     * @param example
     */
    example(example) {
        this.globalCommand.example(example);
        return this;
    }
    outputHelp(subCommand) {
        if (subCommand && this.matchedCommand) {
            this.matchedCommand.outputHelp({
                bin: this.bin,
                subCommands: this.matchedCommand.name === '' ? this.commands : undefined,
                versionNumber: this.globalCommand.versionNumber,
                globalOptions: this.globalCommand.options
            });
        }
        else {
            this.globalCommand.outputHelp({
                bin: this.bin,
                subCommands: this.commands
            });
        }
        return this;
    }
    outputVersion() {
        this.globalCommand.outputVersion(this.bin);
        return this;
    }
    parse(argv = process.argv) {
        this.rawArgs = argv;
        this.bin = argv[1] ? path_1.default.basename(argv[1]) : 'cli';
        // Search sub-commands
        for (const command of this.commands) {
            const minimistOptions = utils_1.getMinimistOptions(this.globalCommand, command);
            const { args, options, originalOptions } = this.minimist(argv.slice(2), minimistOptions);
            const commandName = args[0];
            if (command.isMatched(commandName)) {
                this.matchedCommand = command;
                this.args = args.slice(1);
                this.options = options;
                this.emit(`command:${commandName}`, command);
                this.runCommandAction(command, this.globalCommand, {
                    args: this.args,
                    options,
                    originalOptions
                });
                return { args: this.args, options };
            }
        }
        // Search the default command
        for (const command of this.commands) {
            if (command.name === '') {
                const minimistOptions = utils_1.getMinimistOptions(this.globalCommand, command);
                const { args, options, originalOptions } = this.minimist(argv.slice(2), minimistOptions);
                this.matchedCommand = command;
                this.args = args;
                this.options = options;
                this.emit(`command:!`, command);
                this.runCommandAction(command, this.globalCommand, {
                    args,
                    options,
                    originalOptions
                });
                return { args, options };
            }
        }
        const globalMinimistOptions = utils_1.getMinimistOptions(this.globalCommand);
        const { args, options } = this.minimist(argv.slice(2), globalMinimistOptions);
        this.args = args;
        this.options = options;
        if (options.help && this.globalCommand.hasOption('help')) {
            this.outputHelp();
        }
        if (options.version &&
            this.globalCommand.hasOption('version') &&
            this.globalCommand.versionNumber) {
            this.outputVersion();
        }
        this.emit('command:*');
        return { args, options };
    }
    minimist(argv, minimistOptions) {
        const parsed = minimist_1.default(argv, Object.assign({
            '--': true
        }, minimistOptions));
        const args = parsed._;
        delete parsed._;
        const options = {};
        for (const key of Object.keys(parsed)) {
            options[utils_1.camelcase(key)] = parsed[key];
        }
        return {
            args,
            options,
            originalOptions: parsed
        };
    }
    runCommandAction(command, globalCommand, { args, options, originalOptions }) {
        if (options.help && globalCommand.hasOption('help')) {
            return this.outputHelp(true);
        }
        if (options.version && globalCommand.hasOption('version')) {
            return this.outputVersion();
        }
        if (!command.commandAction)
            return;
        command.checkUnknownOptions(originalOptions, globalCommand);
        command.checkRequiredOptions(originalOptions, globalCommand);
        const minimalArgsCount = command.args.filter(arg => arg.required).length;
        if (args.length < minimalArgsCount) {
            console.error(`error: missing required args for command \`${command.rawName}\``);
            process.exit(1);
        }
        const actionArgs = [];
        command.args.forEach((arg, index) => {
            if (arg.variadic) {
                actionArgs.push(args.slice(index));
            }
            else {
                actionArgs.push(args[index]);
            }
        });
        actionArgs.push(options);
        return command.commandAction.apply(this, actionArgs);
    }
}
const cac = () => new CAC();
module.exports = cac;
